@@ -6,7 +6,7 @@ try:
     from matplotlib import artist
     from matplotlib.patches import FancyArrowPatch
     from mpl_toolkits.mplot3d import proj3d
-    from mpl_toolkits.mplot3d.art3d import Line3D, Text3D, Poly3DCollection
+    from mpl_toolkits.mplot3d.art3d import Line3D, Text3D, Poly3DCollection, Line3DCollection
     from .transformations import transform
     from .rotations import unitx, unitz, perpendicular_to_vectors, norm_vector
 
@@ -651,7 +651,7 @@ try:
         return ax
 
 
-    def plot_mesh(ax=None, filename=None, A2B=np.eye(4), s=np.array([1.0, 1.0, 1.0]), ax_s=1, convex_hull=False, alpha=1.0, color="k"):
+    def plot_mesh(ax=None, filename=None, A2B=np.eye(4), s=np.array([1.0, 1.0, 1.0]), ax_s=1, wireframe=False, convex_hull=False, alpha=1.0, color="k"):
         """Plot mesh.
 
         Note that this function requires the additional library 'trimesh'. It will
@@ -674,6 +674,9 @@ try:
         ax_s : float, optional (default: 1)
             Scaling of the new matplotlib 3d axis
 
+        wireframe : bool, optional (default: True)
+            Plot wireframe of mesh and surface otherwise
+
         convex_hull : bool, optional (default: False)
             Show convex hull instead of the original mesh. This can be much
             faster.
@@ -712,11 +715,39 @@ try:
         vertices = np.hstack((vertices, np.ones((len(vertices), 1))))
         vertices = transform(A2B, vertices)[:, :3]
         vectors = np.array([vertices[[i, j, k]] for i, j, k in mesh.faces])
-        p3c = Poly3DCollection(vectors)
-        p3c.set_alpha(alpha)
-        p3c.set_facecolor(color)
-        ax.add_collection3d(p3c)
+        if wireframe:
+            surface = Line3DCollection(vectors)
+            surface.set_color(color)
+        else:
+            surface = Poly3DCollection(vectors)
+            surface.set_facecolor(color)
+        surface.set_alpha(alpha)
+        ax.add_collection3d(surface)
         return ax
 
+
+    def remove_frame(ax, left=0.0, bottom=0.0, right=1.0, top=1.0):
+        """Remove axis and scale bbox.
+
+        Parameters
+        ----------
+        ax : Matplotlib 3d axis
+            Axis from which we remove the frame
+
+        left : float, optional (default: 0)
+            Position of left border (between 0 and 1)
+
+        bottom : float, optional (default: 0)
+            Position of bottom border (between 0 and 1)
+
+        right : float, optional (default: 1)
+            Position of right border (between 0 and 1)
+
+        top : float, optional (default: 1)
+            Position of top border (between 0 and 1)
+        """
+        ax.axis("off")
+        plt.subplots_adjust(left=left, bottom=bottom, right=right, top=top)
+
 except ImportError:
     warnings.warn("Matplotlib is not installed, visualization is not available")
