@@ -1,4 +1,7 @@
-"""Load transformations from URDF files."""
+"""Load transformations from URDF files.
+
+See :doc:`transform_manager` for more information.
+"""
 import os
 import numpy as np
 from bs4 import BeautifulSoup
@@ -40,6 +43,8 @@ class UrdfTransformManager(TransformManager):
         self._joints = {}
         self.collision_objects = []
         self.visuals = []
+        self.mesh_path = None
+        self.package_dir = None
 
     def add_joint(self, joint_name, from_frame, to_frame, child2parent, axis,
                   limits=(float("-inf"), float("inf")), joint_type="revolute"):
@@ -123,7 +128,7 @@ class UrdfTransformManager(TransformManager):
             raise KeyError("Joint '%s' is not known" % joint_name)
         return self._joints[joint_name][4]
 
-    def load_urdf(self, urdf_xml, mesh_path=None):
+    def load_urdf(self, urdf_xml, mesh_path=None, package_dir=None):
         """Load URDF file into transformation manager.
 
         Parameters
@@ -133,9 +138,16 @@ class UrdfTransformManager(TransformManager):
 
         mesh_path : str, optional (default: None)
             Path in which we search for meshes that are defined in the URDF.
-            Meshes will be ignored if it is set to None.
+            Meshes will be ignored if it is set to None and no 'package_dir'
+            is given.
+
+        package_dir : str, optional (default: None)
+            Some URDFs start file names with 'package://' to refer to the ROS
+            package in which these files (textures, meshes) are located. This
+            variable defines to which path this prefix will be resolved.
         """
         self.mesh_path = mesh_path
+        self.package_dir = package_dir
         urdf = BeautifulSoup(urdf_xml, "xml")
 
         # URDF XML schema:
@@ -239,7 +251,9 @@ class UrdfTransformManager(TransformManager):
             shapes = geometry.findAll(shape_type)
             Cls = shape_classes[shape_type]
             for shape in shapes:
-                shape_object = Cls(name, mesh_path=self.mesh_path, color=color)
+                shape_object = Cls(
+                    name, mesh_path=self.mesh_path,
+                    package_dir=self.package_dir, color=color)
                 shape_object.parse(shape)
                 result.append(shape_object)
         return result
@@ -329,7 +343,7 @@ class UrdfTransformManager(TransformManager):
                 upper = float(limit["upper"])
         return lower, upper
 
-    def plot_visuals(self, frame, ax=None, ax_s=1):
+    def plot_visuals(self, frame, ax=None, ax_s=1, wireframe=False, convex_hull_of_mesh=True, alpha=0.3):
         """Plot all visuals in a given reference frame.
 
         Visuals can be boxes, spheres, cylinders, or meshes. Note that visuals
@@ -346,14 +360,25 @@ class UrdfTransformManager(TransformManager):
         ax_s : float, optional (default: 1)
             Scaling of the new matplotlib 3d axis
 
+        wireframe : bool, optional (default: False)
+            Plot wireframe (surface otherwise)
+
+        convex_hull_of_mesh : bool, optional (default: True)
+            Displays convex hull of meshes instead of the original mesh. This
+            makes plotting a lot faster with complex meshes.
+
+        alpha : float, optional (default: 0.3)
+            Alpha value of the surface / wireframe that will be plotted
+
         Returns
         -------
         ax : Matplotlib 3d axis
             New or old axis
         """
-        return self._plot_objects(self.visuals, frame, ax, ax_s)
+        return self._plot_objects(
+            self.visuals, frame, ax, ax_s, wireframe, convex_hull_of_mesh, alpha)
 
-    def plot_collision_objects(self, frame, ax=None, ax_s=1):
+    def plot_collision_objects(self, frame, ax=None, ax_s=1, wireframe=True, convex_hull_of_mesh=True, alpha=1.0):
         """Plot all collision objects in a given reference frame.
 
         Collision objects can be boxes, spheres, cylinders, or meshes. Note
@@ -371,14 +396,25 @@ class UrdfTransformManager(TransformManager):
         ax_s : float, optional (default: 1)
             Scaling of the new matplotlib 3d axis
 
+        wireframe : bool, optional (default: True)
+            Plot wireframe (surface otherwise)
+
+        convex_hull_of_mesh : bool, optional (default: True)
+            Displays convex hull of meshes instead of the original mesh. This
+            makes plotting a lot faster with complex meshes.
+
+        alpha : float, optional (default: 1)
+            Alpha value of the surface / wireframe that will be plotted
+
         Returns
         -------
         ax : Matplotlib 3d axis
             New or old axis
         """
-        return self._plot_objects(self.collision_objects, frame, ax, ax_s)
+        return self._plot_objects(
+            self.collision_objects, frame, ax, ax_s, wireframe, convex_hull_of_mesh, alpha)
 
-    def _plot_objects(self, objects, frame, ax=None, ax_s=1):
+    def _plot_objects(self, objects, frame, ax=None, ax_s=1, wireframe=True, convex_hull_of_mesh=True, alpha=1.0):
         """Plot all objects in a given reference frame.
 
         Objects can be boxes, spheres, cylinders, or meshes. Note that objects
@@ -386,6 +422,9 @@ class UrdfTransformManager(TransformManager):
 
         Parameters
         ----------
+        objects : list
+            Objects that will be plotted
+
         frame : string
             Reference frame
 
@@ -395,6 +434,16 @@ class UrdfTransformManager(TransformManager):
         ax_s : float, optional (default: 1)
             Scaling of the new matplotlib 3d axis
 
+        wireframe : bool, optional (default: True)
+            Plot wireframe (surface otherwise)
+
+        convex_hull_of_mesh : bool, optional (default: True)
+            Displays convex hull of meshes instead of the original mesh. This
+            makes plotting a lot faster with complex meshes.
+
+        alpha : float, optional (default: 1)
+            Alpha value of the surface / wireframe that will be plotted
+
         Returns
         -------
         ax : Matplotlib 3d axis
@@ -403,7 +452,9 @@ class UrdfTransformManager(TransformManager):
         if ax is None:
             ax = make_3d_axis(ax_s)
         for obj in objects:
-            ax = obj.plot(self, frame, ax)
+            ax = obj.plot(
+                self, frame, ax, wireframe=wireframe,
+                convex_hull=convex_hull_of_mesh, alpha=alpha)
         return ax
 
 
@@ -454,41 +505,47 @@ class Joint(object):
 
 
 class Box(object):
-    def __init__(self, frame, mesh_path, color):
+    def __init__(self, frame, mesh_path, package_dir, color):
         self.frame = frame
         self.color = color
+        self.size = np.zeros(3)
 
     def parse(self, box):
-        self.size = np.zeros(3)
         if box.has_attr("size"):
-            self.size = np.fromstring(box["size"], sep=" ")
+            self.size[:] = np.fromstring(box["size"], sep=" ")
 
-    def plot(self, tm, frame, ax=None, wireframe=True):
+    def plot(self, tm, frame, ax=None, alpha=0.3, wireframe=True, convex_hull=True):
         A2B = tm.get_transform(self.frame, frame)
         color = self.color if self.color is not None else "k"
-        return plot_box(ax, self.size, A2B, wireframe=wireframe, color=color)
+        return plot_box(
+            ax, self.size, A2B, wireframe=wireframe, alpha=alpha, color=color)
 
 
 class Sphere(object):
-    def __init__(self, frame, mesh_path, color):
+    def __init__(self, frame, mesh_path, package_dir, color):
         self.frame = frame
         self.color = color
+        self.radius = 0.0
 
     def parse(self, sphere):
         if not sphere.has_attr("radius"):
             raise UrdfException("Sphere has no radius.")
         self.radius = float(sphere["radius"])
 
-    def plot(self, tm, frame, ax=None, wireframe=True):
+    def plot(self, tm, frame, ax=None, alpha=0.3, wireframe=True, convex_hull=True):
         center = tm.get_transform(self.frame, frame)[:3, 3]
         color = self.color if self.color is not None else "k"
-        return plot_sphere(ax, self.radius, center, wireframe=wireframe, color=color)
+        return plot_sphere(
+            ax, self.radius, center, wireframe=wireframe, alpha=alpha,
+            color=color)
 
 
 class Cylinder(object):
-    def __init__(self, frame, mesh_path, color):
+    def __init__(self, frame, mesh_path, package_dir, color):
         self.frame = frame
         self.color = color
+        self.radius = 0.0
+        self.length = 0.0
 
     def parse(self, cylinder):
         if not cylinder.has_attr("radius"):
@@ -498,36 +555,44 @@ class Cylinder(object):
             raise UrdfException("Cylinder has no length.")
         self.length = float(cylinder["length"])
 
-    def plot(self, tm, frame, ax=None, wireframe=True):
+    def plot(self, tm, frame, ax=None, alpha=0.3, wireframe=True, convex_hull=True):
         A2B = tm.get_transform(self.frame, frame)
         color = self.color if self.color is not None else "k"
-        return plot_cylinder(ax, self.length, self.radius, 0.0, A2B, wireframe, color)
+        return plot_cylinder(
+            ax, self.length, self.radius, 0.0, A2B, wireframe=wireframe,
+            alpha=alpha, color=color)
 
 
 class Mesh(object):
-    def __init__(self, frame, mesh_path, color):
+    def __init__(self, frame, mesh_path, package_dir, color):
         self.frame = frame
         self.mesh_path = mesh_path
+        self.package_dir = package_dir
         self.color = color
+        self.filename = None
+        self.scale = np.ones(3)
 
     def parse(self, mesh):
-        if self.mesh_path is None:
+        if self.mesh_path is None and self.package_dir is None:
             self.filename = None
-            self.scale = 1.0
         else:
             if not mesh.has_attr("filename"):
                 raise UrdfException("Mesh has no filename.")
-            self.filename = mesh["filename"]
-            self.filename = os.path.join(self.mesh_path, self.filename)
+            if self.mesh_path is not None:
+                self.filename = os.path.join(self.mesh_path, mesh["filename"])
+            else:
+                assert self.package_dir is not None
+                self.filename = mesh["filename"].replace(
+                    "package://", self.package_dir)
             if mesh.has_attr("scale"):
                 self.scale = np.fromstring(mesh["scale"], sep=" ")
-            else:
-                self.scale = np.ones(3)
 
-    def plot(self, tm, frame, ax=None, alpha=0.3, convex_hull=True):
+    def plot(self, tm, frame, ax=None, alpha=0.3, wireframe=True, convex_hull=True):
         A2B = tm.get_transform(self.frame, frame)
         color = self.color if self.color is not None else "k"
-        return plot_mesh(ax, self.filename, A2B, self.scale, convex_hull=convex_hull, alpha=alpha, color=color)
+        return plot_mesh(
+            ax, self.filename, A2B, self.scale, wireframe=wireframe,
+            convex_hull=convex_hull, alpha=alpha, color=color)
 
 
 shape_classes = {"box": Box,
